% check.m: Checks solution found by solv.m for the model of money's
%            role by making sure that the equilibrium conditions
%            hold after each type of shock. If the solution is
%            accurate, this program will return a 13x4 matrix of zeros.
%
% THIS PROGRAM WAS WRITTEN FOR MATLAB BY
%
%   PETER N. IRELAND
%   BOSTON COLLEGE
%   DEPARTMENT OF ECONOMICS
%   140 COMMONWEALTH AVENUE
%   CHESTNUT HILL, MA 02467-3806
%   irelandp@bc.edu
%
%  FINANCIAL SUPPORT FROM THE NATIONAL SCIENCE FOUNDATION UNDER
%    GRANT NO. SES-9985763 IS GRATEFULLY ACKNOWLEDGED.

% check equilibrium conditions after an IS shock

  evec = [ siga 0 0 0 ]';

  st = bigw*evec;
  ft = bigu*st;

  ylag = st(1);
  mlag = st(2);
  plag = st(3);
  rlag = st(4);
  mulag = st(5);
  at = st(6);
  et = st(7);
  zt = st(8);
  ert = st(9);

  mt = ft(1);
  rt = ft(2);
  mut = ft(3);
  yt = ft(4);
  pt = ft(5);

  stp = bigpi*st;
  ftp = bigu*stp;

  ylagp = stp(1);
  mlagp = stp(2);
  plagp = stp(3);
  rlagp = stp(4);
  mulagp = stp(5);
  atp = stp(6);
  etp = stp(7);
  ztp = stp(8);
  ertp = stp(9);

  mtp = ftp(1);
  rtp = ftp(2);
  mutp = ftp(3);
  ytp = ftp(4);
  ptp = ftp(5);

  eqconda = ones(13,1);

  eqconda(1) = atp - rhoa*at;

  eqconda(2) = etp - rhoe*et;

  eqconda(3) = yt - ytp + omega1*(rt-ptp) - omega2*(mt-mtp) ...
                 + omega2*(et-etp) - omega1*(at-atp);

  eqconda(4) = mt - gamma1*yt + gamma2*rt - gamma3*et;

  eqconda(5) = ztp  - rhoz*zt;

  eqconda(6) = pt - (pss/rss)*ptp ...
                 - psi*((1/omega1)*yt-(omega2/omega1)*(mt-et)-zt);

  eqconda(7) = mlag + mut - mt - pt;

  eqconda(8) = rt - rhor*rlag ...
                 - rhoy*ylag - rhop*plag - rhom*mulag - ert;

  eqconda(9) = yt - ylagp;

  eqconda(10) = mt - mlagp;

  eqconda(11) = pt - plagp;

  eqconda(12) = rt - rlagp;

  eqconda(13) = mut - mulagp;

% check equilibrium conditions after an LM shock

  evec = [ 0 sige 0 0 ]';

  st = bigw*evec;
  ft = bigu*st;

  ylag = st(1);
  mlag = st(2);
  plag = st(3);
  rlag = st(4);
  mulag = st(5);
  at = st(6);
  et = st(7);
  zt = st(8);
  ert = st(9);

  mt = ft(1);
  rt = ft(2);
  mut = ft(3);
  yt = ft(4);
  pt = ft(5);

  stp = bigpi*st;
  ftp = bigu*stp;

  ylagp = stp(1);
  mlagp = stp(2);
  plagp = stp(3);
  rlagp = stp(4);
  mulagp = stp(5);
  atp = stp(6);
  etp = stp(7);
  ztp = stp(8);
  ertp = stp(9);

  mtp = ftp(1);
  rtp = ftp(2);
  mutp = ftp(3);
  ytp = ftp(4);
  ptp = ftp(5);

  eqconde = ones(13,1);

  eqconde(1) = atp - rhoa*at;

  eqconde(2) = etp - rhoe*et;

  eqconde(3) = yt - ytp + omega1*(rt-ptp) - omega2*(mt-mtp) ...
                 + omega2*(et-etp) - omega1*(at-atp);

  eqconde(4) = mt - gamma1*yt + gamma2*rt - gamma3*et;

  eqconde(5) = ztp  - rhoz*zt;

  eqconde(6) = pt - (pss/rss)*ptp ...
                 - psi*((1/omega1)*yt-(omega2/omega1)*(mt-et)-zt);

  eqconde(7) = mlag + mut - mt - pt;

  eqconde(8) = rt - rhor*rlag ...
                 - rhoy*ylag - rhop*plag - rhom*mulag - ert;

  eqconde(9) = yt - ylagp;

  eqconde(10) = mt - mlagp;

  eqconde(11) = pt - plagp;

  eqconde(12) = rt - rlagp;

  eqconde(13) = mut - mulagp;

% check equilibrium conditions after a supply shock

  evec = [ 0 0 sigz 0 ]';

  st = bigw*evec;
  ft = bigu*st;

  ylag = st(1);
  mlag = st(2);
  plag = st(3);
  rlag = st(4);
  mulag = st(5);
  at = st(6);
  et = st(7);
  zt = st(8);
  ert = st(9);

  mt = ft(1);
  rt = ft(2);
  mut = ft(3);
  yt = ft(4);
  pt = ft(5);

  stp = bigpi*st;
  ftp = bigu*stp;

  ylagp = stp(1);
  mlagp = stp(2);
  plagp = stp(3);
  rlagp = stp(4);
  mulagp = stp(5);
  atp = stp(6);
  etp = stp(7);
  ztp = stp(8);
  ertp = stp(9);

  mtp = ftp(1);
  rtp = ftp(2);
  mutp = ftp(3);
  ytp = ftp(4);
  ptp = ftp(5);

  eqcondz = ones(13,1);

  eqcondz(1) = atp - rhoa*at;

  eqcondz(2) = etp - rhoe*et;

  eqcondz(3) = yt - ytp + omega1*(rt-ptp) - omega2*(mt-mtp) ...
                 + omega2*(et-etp) - omega1*(at-atp);

  eqcondz(4) = mt - gamma1*yt + gamma2*rt - gamma3*et;

  eqcondz(5) = ztp  - rhoz*zt;

  eqcondz(6) = pt - (pss/rss)*ptp ...
                 - psi*((1/omega1)*yt-(omega2/omega1)*(mt-et)-zt);

  eqcondz(7) = mlag + mut - mt - pt;

  eqcondz(8) = rt - rhor*rlag ...
                 - rhoy*ylag - rhop*plag - rhom*mulag - ert;

  eqcondz(9) = yt - ylagp;

  eqcondz(10) = mt - mlagp;

  eqcondz(11) = pt - plagp;

  eqcondz(12) = rt - rlagp;

  eqcondz(13) = mut - mulagp;

% check equilibrium conditions after a policy shock

  evec = [ 0 0 0 sigr ]';

  st = bigw*evec;
  ft = bigu*st;

  ylag = st(1);
  mlag = st(2);
  plag = st(3);
  rlag = st(4);
  mulag = st(5);
  at = st(6);
  et = st(7);
  zt = st(8);
  ert = st(9);

  mt = ft(1);
  rt = ft(2);
  mut = ft(3);
  yt = ft(4);
  pt = ft(5);

  stp = bigpi*st;
  ftp = bigu*stp;

  ylagp = stp(1);
  mlagp = stp(2);
  plagp = stp(3);
  rlagp = stp(4);
  mulagp = stp(5);
  atp = stp(6);
  etp = stp(7);
  ztp = stp(8);
  ertp = stp(9);

  mtp = ftp(1);
  rtp = ftp(2);
  mutp = ftp(3);
  ytp = ftp(4);
  ptp = ftp(5);

  eqcondr = ones(13,1);

  eqcondr(1) = atp - rhoa*at;

  eqcondr(2) = etp - rhoe*et;

  eqcondr(3) = yt - ytp + omega1*(rt-ptp) - omega2*(mt-mtp) ...
                 + omega2*(et-etp) - omega1*(at-atp);

  eqcondr(4) = mt - gamma1*yt + gamma2*rt - gamma3*et;

  eqcondr(5) = ztp  - rhoz*zt;

  eqcondr(6) = pt - (pss/rss)*ptp ...
                 - psi*((1/omega1)*yt-(omega2/omega1)*(mt-et)-zt);

  eqcondr(7) = mlag + mut - mt - pt;

  eqcondr(8) = rt - rhor*rlag ...
                 - rhoy*ylag - rhop*plag - rhom*mulag - ert;

  eqcondr(9) = yt - ylagp;

  eqcondr(10) = mt - mlagp;

  eqcondr(11) = pt - plagp;

  eqcondr(12) = rt - rlagp;

  eqcondr(13) = mut - mulagp;

% report results

  [ eqconda eqconde eqcondz eqcondr ]