% dcorr4.m: Computes vector autocorrelations implied by an
%             unconstrained VAR with four lags of m, y, p, and r. 
%
%           Produces 16 vectors of the form dcorrij where i and j
%             are drawn from the set {y,m,p,r}.  Each dcorrij is 
%             a 41x1 vector with kth element given by
%
%                  cov(i(t),j(t-k))/[std(i)*std(j)].
%
% THIS PROGRAM WAS WRITTEN FOR MATLAB BY
%
%   PETER N. IRELAND
%   BOSTON COLLEGE
%   DEPARTMENT OF ECONOMICS
%   140 COMMONWEALTH AVENUE
%   CHESTNUT HILL, MA 02467-3806
%   irelandp@bc.edu
%
%  FINANCIAL SUPPORT FROM THE NATIONAL SCIENCE FOUNDATION UNDER
%    GRANT NO. SES-9985763 IS GRATEFULLY ACKNOWLEDGED.

% load data, choose sample period, detrend, and demean

  global yt mt pt rt

  load ympr2.dat;

  yt = ympr2(84:170,1);
  mt = ympr2(84:170,2);
  pt = ympr2(84:170,3);
  rt = ympr2(84:170,4);

  bigt = length(yt);

  trend = 1:bigt;

  yt = yt - 0.005074*trend';
  mt = mt - 0.002644*trend';

  yt = yt - mean(yt);
  mt = mt - mean(mt);
  pt = pt - mean(pt);
  rt = rt - mean(rt);

  dt = [ yt(5:87) mt(5:87) pt(5:87) rt(5:87) ];

  dlag1 = [ yt(4:86) mt(4:86) pt(4:86) rt(4:86) ];
  dlag2 = [ yt(3:85) mt(3:85) pt(3:85) rt(3:85) ];
  dlag3 = [ yt(2:84) mt(2:84) pt(2:84) rt(2:84) ];
  dlag4 = [ yt(1:83) mt(1:83) pt(1:83) rt(1:83) ];

  dlag = [ dlag1 dlag2 dlag3 dlag4 ];

  bigt = 83;

% estimate the VAR

  bigbetp = inv(dlag'*dlag)*dlag'*dt;

  bigbet = bigbetp';

  epst = dt - dlag*bigbetp;

  bigv = (1/bigt)*epst'*epst;

% form companion matrices A and B

  biga = [ bigbet ; ...
           eye(4) zeros(4,12) ; ...
           zeros(4,4) eye(4) zeros(4,8) ; ...
           zeros(4,8) eye(4) zeros(4,4) ];

  bigb = [ eye(4) ; zeros(12,4) ];

  bigbvb = bigb*bigv*bigb';

% compute autocorrelations

  dcorrmat = zeros(16,41);

  bigsigd = inv(eye(256)-kron(biga,biga))*bigbvb(:);

  bigsigd = reshape(bigsigd,16,16);

  stdvec = sqrt(diag(bigsigd));

  stdmat = stdvec*stdvec';

  bigak = eye(16);

  for k = 0:40

    biggamk = bigak*bigsigd;

    biggamk = biggamk./stdmat;

    biggamk = biggamk(1:4,1:4);

    biggamk = biggamk';

    dcorrmat(:,k+1) = biggamk(:);

    bigak = biga*bigak;

  end

% create output vectors

  dcorryy = dcorrmat(1,:)';
  dcorrym = dcorrmat(2,:)';
  dcorryp = dcorrmat(3,:)';
  dcorryr = dcorrmat(4,:)';
  dcorrmy = dcorrmat(5,:)';
  dcorrmm = dcorrmat(6,:)';
  dcorrmp = dcorrmat(7,:)';
  dcorrmr = dcorrmat(8,:)';
  dcorrpy = dcorrmat(9,:)';
  dcorrpm = dcorrmat(10,:)';
  dcorrpp = dcorrmat(11,:)';
  dcorrpr = dcorrmat(12,:)';
  dcorrry = dcorrmat(13,:)';
  dcorrrm = dcorrmat(14,:)';
  dcorrrp = dcorrmat(15,:)';
  dcorrrr = dcorrmat(16,:)';