% est.m: Maximizes the (minimizes the negative) log likelihood function
%          for the model of money's role.
%
%        When maximizing the log likelihood function, the parameters
%          are transformed to satisfy theoretical restrictions.
%          The log likelihood function with transformed parameters
%          is contained in llfn.m.
%
% THIS PROGRAM WAS WRITTEN FOR MATLAB BY
%
%   PETER N. IRELAND
%   BOSTON COLLEGE
%   DEPARTMENT OF ECONOMICS
%   140 COMMONWEALTH AVENUE
%   CHESTNUT HILL, MA 02467-3806
%   irelandp@bc.edu
%
%  FINANCIAL SUPPORT FROM THE NATIONAL SCIENCE FOUNDATION UNDER
%    GRANT NO. SES-9985763 IS GRATEFULLY ACKNOWLEDGED.

% load and detrend M2 data

%  global yt mt pt rt

%  load ympr2.dat;

%  yt = ympr2(84:170,1);
%  mt = ympr2(84:170,2);
%  pt = ympr2(84:170,3);
%  rt = ympr2(84:170,4);

%  bigt = length(yt);

%  trend = 1:bigt;

%  yt = yt - 0.005074*trend';
%  mt = mt - 0.002644*trend';

% load and detrend M1 data

  global yt mt pt rt

  load ympr1.dat;

  yt = ympr1(84:170,1);
  mt = ympr1(84:170,2);
  pt = ympr1(84:170,3);
  rt = ympr1(84:170,4);

  bigt = length(yt);

  trend = 1:bigt;

  yt = yt - 0.005074*trend';
  mt = mt - 0.003323*trend';

% set starting values for M2, omega2 estimated

%  omega1tr = 1;
%  omega2tr = 0.1;
%  gamma2tr = 1;
%  psitr = 0.1;
%  rhortr = 0.75;
%  rhoytr = 0.05;
%  rhoptr = 0.35;
%  rhomtr = 0.00000001;
%  lnysstr = 8.8619;
%  lnmsstr = 9.7248;
%  lnpsstr = 0.0082;
%  lnrsstr = 0.0169;
%  rhoatr = 0.3;
%  rhoetr = 0.3;
%  rhoztr = 0.3;
%  siga = 0.0293;
%  sige = 0.0101;
%  sigz = 0.0083;
%  sigr = 0.0025;

%  bigtheto = [ omega2tr gamma2tr ...
%               rhortr rhoytr rhoptr ...
%               lnysstr lnmsstr lnpsstr lnrsstr ...
%               rhoatr rhoetr rhoztr ...
%               siga sige sigz sigr ]';

% set starting values for M2, omega2 fixed

%  omega1tr = 1;
%  omega2tr = 0.25;
%  gamma2tr = 1;
%  psitr = 0.1;
%  rhortr = 0.75;
%  rhoytr = 0.05;
%  rhoptr = 0.35;
%  rhomtr = 0.00000001;
%  lnysstr = 8.8619;
%  lnmsstr = 9.7248;
%  lnpsstr = 0.0082;
%  lnrsstr = 0.0169;
%  rhoatr = 0.3;
%  rhoetr = 0.3;
%  rhoztr = 0.3;
%  siga = 0.0293;
%  sige = 0.0100;
%  sigz = 0.0083;
%  sigr = 0.0025;

%  bigtheto = [ gamma2tr ...
%               rhortr rhoytr rhoptr ...
%               lnysstr lnmsstr lnpsstr lnrsstr ...
%               rhoatr rhoetr rhoztr ...
%               siga sige sigz sigr ]';

% set starting values for M1, omega2 estimated

  omega1tr = 1;
  omega2tr = 0.1;
  gamma2tr = 1;
  psitr = 0.1;
  rhortr = 0.75;
  rhoytr = 0.05;
  rhoptr = 0.35;
  rhomtr = 0.00000001;
  lnysstr = 8.8619;
  lnmsstr = 8.3579;
  lnpsstr = 0.0082;
  lnrsstr = 0.0169;
  rhoatr = 0.3;
  rhoetr = 0.3;
  rhoztr = 0.3;
  siga = 0.0292;
  sige = 0.0173;
  sigz = 0.0082;
  sigr = 0.0025;

  bigtheto = [ omega2tr gamma2tr ...
               rhortr rhoytr rhoptr ...
               lnysstr lnmsstr lnpsstr lnrsstr ...
               rhoatr rhoetr rhoztr ...
               siga sige sigz sigr ]';

% maximize likelihood

  options(1) = 1;
  options(14) = 10000;

  thetstar = fminu('llfn',bigtheto,options);