% solv.m: Solves the model of money's role.
%
% THIS PROGRAM WAS WRITTEN FOR MATLAB BY
%
%   PETER N. IRELAND
%   BOSTON COLLEGE
%   DEPARTMENT OF ECONOMICS
%   140 COMMONWEALTH AVENUE
%   CHESTNUT HILL, MA 02467-3806
%   irelandp@bc.edu
%
%  FINANCIAL SUPPORT FROM THE NATIONAL SCIENCE FOUNDATION UNDER
%    GRANT NO. SES-9985763 IS GRATEFULLY ACKNOWLEDGED.

% set parameter values

%  omega1 = 1;
%  omega2 = 0.0000;
%  gamma2 = 0.7220;
%  psi = 0.1;
%  rhor = 0.5481;
%  rhoy = 0.0000;
%  rhop = 0.5680;
%  rhom = 0.00000001;
%  lnyss = 8.8731;
%  lnmss = 9.7381;
%  lnpss = 0.0102;
%  lnrss = 0.0189;
%  rhoa = 0.9579;
%  rhoe = 0.9853;
%  rhoz = 0.9903;
%  siga = 0.0188;
%  sige = 0.0088;
%  sigz = 0.0098;
%  sigr = 0.0025;

  omega1 = 1;
  omega2 = 0.25;
  gamma2 = 0.1251;
  psi = 0.1;
  rhor = 0.5481;
  rhoy = 0.0000;
  rhop = 0.5680;
  rhom = 0.00000001;
  lnyss = 8.8747;
  lnmss = 9.7383;
  lnpss = 0.0100;
  lnrss = 0.0186;
  rhoa = 0.9575;
  rhoe = 0.9867;
  rhoz = 0.9904;
  siga = 0.0187;
  sige = 0.0088;
  sigz = 0.0098;
  sigr = 0.0025;

  yss = exp(lnyss);
  mss = exp(lnmss);
  pss = exp(lnpss);
  rss = exp(lnrss);

  gamma1 = (rss-1+yss*rss*omega2/mss)*(gamma2/omega1);
  gamma3 = 1 - (rss-1)*gamma2;

% form matrices A, B, and C

  biga = zeros(3,3);
  bigb = zeros(3,7);
  bigc = zeros(3,4);

  biga(1,1) = 1;
  biga(1,2) = gamma2;
  bigb(1,6) = gamma1;
  bigc(1,2) = gamma3;

  biga(2,1) = 1;
  biga(2,3) = -1;
  bigb(2,2) = 1;
  bigb(2,7) = -1;

  biga(3,2) = 1;
  bigb(3,1) = rhoy;
  bigb(3,3) = rhop;
  bigb(3,4) = rhor;
  bigb(3,5) = rhom;
  bigc(3,4) = 1;

% form matrices D, F, G, H, and J

  bigd = zeros(7,7);
  bigf = zeros(7,3);
  bigg = zeros(7,7);
  bigh = zeros(7,3);
  bigj = zeros(7,4);

  bigd(1,6) = 1;
  bigd(1,7) = omega1;
  bigf(1,1) = -omega2;
  bigg(1,6) = 1;
  bigh(1,1) = -omega2;
  bigh(1,2) = omega1;
  bigj(1,1) = -omega1*(1-rhoa);
  bigj(1,2) = omega2*(1-rhoe);

  bigd(2,7) = pss/rss;
  bigg(2,6) = -psi/omega1;
  bigg(2,7) = 1;
  bigh(2,1) = psi*(omega2/omega1);
  bigj(2,2) = -psi*(omega2/omega1);
  bigj(2,3) = psi;

  bigd(3,1) = 1;
  bigg(3,6) = 1;

  bigd(4,2) = 1;
  bigh(4,1) = 1;

  bigd(5,3) = 1;
  bigg(5,7) = 1;

  bigd(6,4) = 1;
  bigh(6,2) = 1;

  bigd(7,5) = 1;
  bigh(7,3) = 1;

% form matrix P

  bigp = zeros(4,4);

  bigp(1,1) = rhoa;
  bigp(2,2) = rhoe;
  bigp(3,3) = rhoz;

% form matrices K and L

  bigkl1 = inv(bigd+bigf*inv(biga)*bigb);
  bigkl2 = bigg + bigh*inv(biga)*bigb;
  bigkl3 = bigj + bigh*inv(biga)*bigc - bigf*inv(biga)*bigc*bigp;

  bigk = bigkl1*bigkl2;

  bigl = bigkl1*bigkl3;

  bigl1 = bigl(1:5,:);
  bigl2 = bigl(6:7,:);

% form matrices M and N

  [kvec,keig] = eig(bigk);

  keig2 = diag(keig);

  [keig3,kord] = sort(abs(keig2));

  if keig3(5) > 1
    'error - no solution'
  end

  if keig3(6) < 1
    'error - multiple solutions'
  end

  keig4 = keig2(kord);

  bign = diag(keig4);

  bign1 = bign(1:5,1:5);
  bign2 = bign(6:7,6:7);

  bigminv = kvec(:,kord);

  bigm = inv(bigminv);

  bigm11 = bigm(1:5,1:5);
  bigm12 = bigm(1:5,6:7);
  bigm21 = bigm(6:7,1:5);
  bigm22 = bigm(6:7,6:7);

% form matrices Q and R

  bigq1 = bigm11*bigl1 + bigm12*bigl2;
  bigq2 = bigm21*bigl1 + bigm22*bigl2;

  vecr = inv(eye(8)-kron(bigp,inv(bign2)))*bigq2(:);

  bigr = reshape(vecr,2,4);

% form matrices S

  bigs1 = -inv(bigm22)*bigm21;

  bigs2 = -inv(bigm22)*inv(bign2)*bigr;

  bigs34 = bigm11 + bigm12*bigs1;

  bigs3 = inv(bigs34)*bign1*bigs34;

  bigs4 = inv(bigs34)*(bigq1+bign1*bigm12*bigs2-bigm12*bigs2*bigp);

  bigs5a = [ eye(5) ; bigs1 ]; 

  bigs5 = inv(biga)*bigb*bigs5a;

  bigs6a = [ zeros(5,4) ; bigs2 ];

  bigs6 = inv(biga)*bigb*bigs6a + inv(biga)*bigc;

% form matrices PI, W, and U

  bigpi = [ bigs3 bigs4 ; zeros(4,5) bigp ];

  bigw = [ zeros(5,4) ; eye(4) ];

  bigu = [ bigs5 bigs6 ; bigs1 bigs2 ];

  bigpi = real(bigpi);

  bigu = real(bigu);